@@ -34,6 +34,12 @@
 #	define	PLATFORM_DOS		1
 #   endif
 
+#elif defined(__OS2__) || defined(__EMX__)
+
+#   if !defined(PLATFORM_OS2)
+#	define	PLATFORM_OS2		1
+#   endif
+
 #elif defined(_WIN32) || defined(_WIN64)
 
 #   if !defined(PLATFORM_WINDOWS)
@@ -52,9 +58,9 @@
 #	define	PLATFORM_MAC		1
 #   endif
 
-#elif defined(__MORPHOS__) || defined(__AROS__)			|| \
-      defined(__amigaos__) || defined(__amigaos4__)		|| \
-      defined(__amigados__) || defined(__AMIGA) || defined(__AMIGA__)
+#elif defined(__MORPHOS__) || defined(__AROS__) || defined(AMIGAOS)	|| \
+      defined(__amigaos__) || defined(__amigados__)			|| \
+      defined(AMIGA) || defined(_AMIGA) || defined(__AMIGA__)
 
 #   if !defined(PLATFORM_AMIGA)
 #	define	PLATFORM_AMIGA		1
@@ -83,7 +89,7 @@
 #   endif
 #endif
 
-#endif	/* end of PLATFORM_ definitions */
+#endif	/* PLATFORM_xxx */
 
 
 #if defined (PLATFORM_OSX)		/* OS X is unix-based */
@@ -108,6 +114,8 @@
 #	define	PLATFORM_STRING	"Windows"
 #elif defined(PLATFORM_DOS)
 #	define	PLATFORM_STRING	"DOS"
+#elif defined(PLATFORM_OS2)
+#	define	PLATFORM_STRING	"OS/2"
 #elif defined(__linux__) || defined(__linux)
 #	define	PLATFORM_STRING	"Linux"
 #elif defined(__DragonFly__)
@@ -147,7 +155,7 @@
 #else
 #	define	PLATFORM_STRING	"Unknown"
 #	warning "Platform is UNKNOWN."
-#endif	/* end of PLATFORM_STRING definitions */
+#endif	/* PLATFORM_STRING */
 
 #endif	/* __ARCH_DEFS__ */
 
