@@ -812,7 +812,7 @@ const char *MSG_ReadString (void)
 	l = 0;
 	do
 	{
-		c = MSG_ReadChar ();
+		c = MSG_ReadByte ();
 		if (c == -1 || c == 0)
 			break;
 		string[l] = c;
