@@ -2,6 +2,7 @@
 Copyright (C) 1996-2001 Id Software, Inc.
 Copyright (C) 2002-2009 John Fitzgibbons and others
 Copyright (C) 2007-2008 Kristian Duske
+Copyright (C) 2010-2014 QuakeSpasm developers
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,7 +35,6 @@ qpic_t		*draw_backtile;
 gltexture_t *char_texture; //johnfitz
 qpic_t		*pic_ovr, *pic_ins; //johnfitz -- new cursor handling
 qpic_t		*pic_nul; //johnfitz -- for missing gfx, don't crash
-qpic_t		*pic_up, *pic_down; //QuakeSpasm -- menu scrolling
 
 //johnfitz -- new pics
 byte pic_ovr_data[8][8] =
@@ -99,32 +99,6 @@ byte pic_crosshair_data[8][8] =
 };
 //johnfitz
 
-//QuakeSpasm -- new pics
-byte pic_up_data[8][8] =
-{
-	{255,255,255,255,255,255,255,255},
-	{255,255,255,255,255,255,255,255},
-	{255,255,255,  7,255,255,255,255},
-	{255,255,  8,  8,  7,255,255,255},
-	{255,  8,  4,  8,  4,  7,255,255},
-	{255,255,  2,  8,  2,  2,  2,255},
-	{255,255,255,  8,  2,255,255,255},
-	{255,255,255,255,  2,255,255,255},
-};
-
-byte pic_down_data[8][8] =
-{
-	{255,255,255,255,255,255,255,255},
-	{255,255,255,255,255,255,255,255},
-	{255,255,255,  8,255,255,255,255},
-	{255,255,255,  8,  2,255,255,255},
-	{255,  8,  4,  8,  4,  7,255,255},
-	{255,255,  8,  8,  7,  2,  2,255},
-	{255,255,255,  7,  2,  2,255,255},
-	{255,255,255,255,  2,255,255,255},
-};
-//QuakeSpasm
-
 typedef struct
 {
 	gltexture_t *gltexture;
@@ -340,66 +314,6 @@ qpic_t	*Draw_CachePic (const char *path)
 	return &pic->pic;
 }
 
-/*
-================
-Draw_ConbackPic -- QuakeSpasm custom conback drawing.
-================
-*/
-#if !defined(USE_QS_CONBACK)
-static inline qpic_t *Draw_ConbackPic (void)
-{
-	return Draw_CachePic ("gfx/conback.lmp");
-}
-#else
-extern char *get_conback(void);
-static qboolean have_mod_conback;
-void Draw_CheckConback (void)
-{
-	have_mod_conback = (COM_LoadTempFile("gfx/conback.lmp", NULL) != NULL);
-}
-qpic_t *Draw_ConbackPic (void)
-{
-    if (fitzmode) {
-	return Draw_CachePic ("gfx/conback.lmp");
-    } else if (have_mod_conback) {
-	/* even if we are running in custom mode
-	   allow for mod-provided conback images */
-	return Draw_CachePic ("gfx/conback.lmp");
-    } else {
-	/* QuakeSpasm customization: */
-	cachepic_t	*pic;
-	int			i;
-	qpic_t		*dat;
-	glpic_t		gl;
-
-	for (pic=menu_cachepics, i=0 ; i<menu_numcachepics ; pic++, i++)
-	{
-		if (!strcmp ("gfx/conback.lmp", pic->name))
-			return &pic->pic;
-	}
-	if (menu_numcachepics == MAX_CACHED_PICS)
-		Sys_Error ("menu_numcachepics == MAX_CACHED_PICS");
-	menu_numcachepics++;
-	strcpy (pic->name, "gfx/conback.lmp");
-	/* load custom conback, image in memory */
-	dat = (qpic_t *)get_conback ();
-	SwapPic (dat);
-	pic->pic.width = dat->width;
-	pic->pic.height = dat->height;
-	gl.gltexture = TexMgr_LoadImage (NULL, "gfx/conback.lmp", dat->width, dat->height, SRC_INDEXED, dat->data,
-					  "", (src_offset_t)dat->data,
-					  TEXPREF_ALPHA | TEXPREF_PAD | TEXPREF_NOPICMIP); //johnfitz -- TexMgr
-	gl.sl = 0;
-	gl.sh = (float)dat->width/(float)TexMgr_PadConditional(dat->width); //johnfitz
-	gl.tl = 0;
-	gl.th = (float)dat->height/(float)TexMgr_PadConditional(dat->height); //johnfitz
-	memcpy (pic->pic.data, &gl, sizeof(glpic_t));
-
-	return &pic->pic;
-    }	/* -- QuakeSpasm */
-}
-#endif	/* USE_QS_CONBACK */
-
 /*
 ================
 Draw_MakePic -- johnfitz -- generate pics from internal data
@@ -493,19 +407,12 @@ void Draw_Init (void)
 	Scrap_Upload (); //creates 2 empty textures
 
 	// create internal pics
-	pic_ins  = Draw_MakePic ("ins",  8, 9, &pic_ins_data[0][0]);
-	pic_ovr  = Draw_MakePic ("ovr",  8, 8, &pic_ovr_data[0][0]);
-	pic_nul  = Draw_MakePic ("nul",  8, 8, &pic_nul_data[0][0]);
-	pic_up   = Draw_MakePic ("up",   8, 8, &pic_up_data[0][0]);
-	pic_down = Draw_MakePic ("down", 8, 8, &pic_down_data[0][0]);
+	pic_ins = Draw_MakePic ("ins", 8, 9, &pic_ins_data[0][0]);
+	pic_ovr = Draw_MakePic ("ovr", 8, 8, &pic_ovr_data[0][0]);
+	pic_nul = Draw_MakePic ("nul", 8, 8, &pic_nul_data[0][0]);
 
 	// load game pics
 	Draw_LoadPics ();
-
-#if defined(USE_QS_CONBACK)
-	/* QuakeSpasm customization: */
-	Draw_CheckConback ();
-#endif	/* USE_QS_CONBACK */
 }
 
 //==============================================================================
@@ -646,7 +553,7 @@ void Draw_ConsoleBackground (void)
 	qpic_t *pic;
 	float alpha;
 
-	pic = Draw_ConbackPic ();
+	pic = Draw_CachePic ("gfx/conback.lmp");
 	pic->width = vid.conwidth;
 	pic->height = vid.conheight;
 
