@@ -62,6 +62,7 @@ cvar_t	r_norefresh = {"r_norefresh","0",CVAR_NONE};
 cvar_t	r_drawentities = {"r_drawentities","1",CVAR_NONE};
 cvar_t	r_drawviewmodel = {"r_drawviewmodel","1",CVAR_NONE};
 cvar_t	r_speeds = {"r_speeds","0",CVAR_NONE};
+cvar_t	r_pos = {"r_pos","0",CVAR_NONE};
 cvar_t	r_fullbright = {"r_fullbright","0",CVAR_NONE};
 cvar_t	r_lightmap = {"r_lightmap","0",CVAR_NONE};
 cvar_t	r_shadows = {"r_shadows","0",CVAR_ARCHIVE};
@@ -103,6 +104,12 @@ extern cvar_t	r_vfog;
 
 cvar_t	gl_zfix = {"gl_zfix", "0", CVAR_NONE}; // QuakeSpasm z-fighting fix
 
+cvar_t	r_lavaalpha = {"r_lavaalpha","0",CVAR_NONE};
+cvar_t	r_telealpha = {"r_telealpha","0",CVAR_NONE};
+cvar_t	r_slimealpha = {"r_slimealpha","0",CVAR_NONE};
+
+float	map_wateralpha, map_lavaalpha, map_telealpha, map_slimealpha;
+
 qboolean r_drawflat_cheatsafe, r_fullbright_cheatsafe, r_lightmap_cheatsafe, r_drawworld_cheatsafe; //johnfitz
 
 //==============================================================================
@@ -113,6 +120,7 @@ qboolean r_drawflat_cheatsafe, r_fullbright_cheatsafe, r_lightmap_cheatsafe, r_d
 
 static GLuint r_gamma_texture;
 static GLuint r_gamma_program;
+static int r_gamma_texture_width, r_gamma_texture_height;
 
 // uniforms used in gamma shader
 static GLuint gammaLoc;
@@ -173,6 +181,8 @@ GLSLGamma_GammaCorrect
 */
 void GLSLGamma_GammaCorrect (void)
 {
+	float smax, tmax;
+
 	if (!gl_glsl_gamma_able)
 		return;
 
@@ -184,8 +194,17 @@ void GLSLGamma_GammaCorrect (void)
 	{
 		glGenTextures (1, &r_gamma_texture);
 		glBindTexture (GL_TEXTURE_2D, r_gamma_texture);
+
+		r_gamma_texture_width = glwidth;
+		r_gamma_texture_height = glheight;
+
+		if (!gl_texture_NPOT)
+		{
+			r_gamma_texture_width = TexMgr_Pad(r_gamma_texture_width);
+			r_gamma_texture_height = TexMgr_Pad(r_gamma_texture_height);
+		}
 	
-		glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA8, glwidth, glheight, 0, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, NULL);
+		glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA8, r_gamma_texture_width, r_gamma_texture_height, 0, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, NULL);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	}
@@ -215,21 +234,24 @@ void GLSLGamma_GammaCorrect (void)
 
 	glViewport (glx, gly, glwidth, glheight);
 
+	smax = glwidth/(float)r_gamma_texture_width;
+	tmax = glheight/(float)r_gamma_texture_height;
+
 	glBegin (GL_QUADS);
 	glTexCoord2f (0, 0);
 	glVertex2f (-1, -1);
-	glTexCoord2f (1, 0);
+	glTexCoord2f (smax, 0);
 	glVertex2f (1, -1);
-	glTexCoord2f (1, 1);
+	glTexCoord2f (smax, tmax);
 	glVertex2f (1, 1);
-	glTexCoord2f (0, 1);
+	glTexCoord2f (0, tmax);
 	glVertex2f (-1, 1);
 	glEnd ();
 	
 	GL_UseProgramFunc (0);
 	
 // clear cached binding
-	GL_Bind (notexture);
+	GL_ClearBindings ();
 }
 
 /*
@@ -579,11 +601,6 @@ void R_SetupView (void)
 //
 //==============================================================================
 
-static int R_EntitySortFunc(const void *a, const void *b)
-{
-	return (*(entity_t **)a)->model - (*(entity_t **)b)->model;
-}
-
 /*
 =============
 R_DrawEntitiesOnList
@@ -591,21 +608,15 @@ R_DrawEntitiesOnList
 */
 void R_DrawEntitiesOnList (qboolean alphapass) //johnfitz -- added parameter
 {
-	entity_t	*entities_sorted[MAX_VISEDICTS];
 	int		i;
 
 	if (!r_drawentities.value)
 		return;
 
-	//ericw -- draw the entities sorted by model, to eliminate redundant texture binding
-	memcpy (entities_sorted, cl_visedicts, cl_numvisedicts * sizeof(entity_t *));
-	qsort (entities_sorted, cl_numvisedicts, sizeof(entity_t *), R_EntitySortFunc);
-	//ericw --
-
 	//johnfitz -- sprites are not a special case
 	for (i=0 ; i<cl_numvisedicts ; i++)
 	{
-		currententity = entities_sorted[i];
+		currententity = cl_visedicts[i];
 
 		//johnfitz -- if alphapass is true, draw only alpha entites this time
 		//if alphapass is false, draw only nonalpha entities this time
@@ -988,7 +999,15 @@ void R_RenderView (void)
 
 	//johnfitz -- modified r_speeds output
 	time2 = Sys_DoubleTime ();
-	if (r_speeds.value == 2)
+	if (r_pos.value)
+		Con_Printf ("x %i y %i z %i (pitch %i yaw %i roll %i)\n",
+			(int)cl_entities[cl.viewentity].origin[0],
+			(int)cl_entities[cl.viewentity].origin[1],
+			(int)cl_entities[cl.viewentity].origin[2],
+			(int)cl.viewangles[PITCH],
+			(int)cl.viewangles[YAW],
+			(int)cl.viewangles[ROLL]);
+	else if (r_speeds.value == 2)
 		Con_Printf ("%3i ms  %4i/%4i wpoly %4i/%4i epoly %3i lmap %4i/%4i sky %1.1f mtex\n",
 					(int)((time2-time1)*1000),
 					rs_brushpolys,
