@@ -77,6 +77,8 @@ int	vec_to_st[6][3] =
 	{-2,1,-3}
 };
 
+float	skyfog; // ericw
+
 //==============================================================================
 //
 //  INIT
@@ -223,6 +225,7 @@ void Sky_NewMap (void)
 	skybox_name[0] = 0;
 	for (i=0; i<6; i++)
 		skybox_textures[i] = NULL;
+	skyfog = r_skyfog.value;
 
 	//
 	// read worldspawn (this is so ugly, and shouldn't it be done on the server?)
@@ -258,6 +261,9 @@ void Sky_NewMap (void)
 		if (!strcmp("sky", key))
 			Sky_LoadSkyBox(value);
 
+		if (!strcmp("skyfog", key))
+			skyfog = atof(value);
+
 #if 1 //also accept non-standard keys
 		else if (!strcmp("skyname", key)) //half-life
 			Sky_LoadSkyBox(value);
@@ -287,6 +293,17 @@ void Sky_SkyCommand_f (void)
 	}
 }
 
+/*
+====================
+R_SetSkyfog_f -- ericw
+====================
+*/
+static void R_SetSkyfog_f (cvar_t *var)
+{
+// clear any skyfog setting from worldspawn
+	skyfog = var->value;
+}
+
 /*
 =============
 Sky_Init
@@ -300,6 +317,7 @@ void Sky_Init (void)
 	Cvar_RegisterVariable (&r_sky_quality);
 	Cvar_RegisterVariable (&r_skyalpha);
 	Cvar_RegisterVariable (&r_skyfog);
+	Cvar_SetCallback (&r_skyfog, R_SetSkyfog_f);
 
 	Cmd_AddCommand ("sky",Sky_SkyCommand_f);
 
@@ -699,14 +717,14 @@ void Sky_DrawSkyBox (void)
 		rs_skypolys++;
 		rs_skypasses++;
 
-		if (Fog_GetDensity() > 0 && r_skyfog.value > 0)
+		if (Fog_GetDensity() > 0 && skyfog > 0)
 		{
 			float *c;
 
 			c = Fog_GetColor();
 			glEnable (GL_BLEND);
 			glDisable (GL_TEXTURE_2D);
-			glColor4f (c[0],c[1],c[2], CLAMP(0.0,r_skyfog.value,1.0));
+			glColor4f (c[0],c[1],c[2], CLAMP(0.0,skyfog,1.0));
 
 			glBegin (GL_QUADS);
 			Sky_EmitSkyBoxVertex (skymins[0][i], skymins[1][i], i);
@@ -850,14 +868,14 @@ void Sky_DrawFaceQuad (glpoly_t *p)
 		rs_skypasses += 2;
 	}
 
-	if (Fog_GetDensity() > 0 && r_skyfog.value > 0)
+	if (Fog_GetDensity() > 0 && skyfog > 0)
 	{
 		float *c;
 
 		c = Fog_GetColor();
 		glEnable (GL_BLEND);
 		glDisable (GL_TEXTURE_2D);
-		glColor4f (c[0],c[1],c[2], CLAMP(0.0,r_skyfog.value,1.0));
+		glColor4f (c[0],c[1],c[2], CLAMP(0.0,skyfog,1.0));
 
 		glBegin (GL_QUADS);
 		for (i=0, v=p->verts[0] ; i<4 ; i++, v+=VERTEXSIZE)
@@ -993,7 +1011,7 @@ void Sky_DrawSky (void)
 	//
 	// render slow sky: cloud layers or skybox
 	//
-	if (!r_fastsky.value && !(Fog_GetDensity() > 0 && r_skyfog.value >= 1))
+	if (!r_fastsky.value && !(Fog_GetDensity() > 0 && skyfog >= 1))
 	{
 		glDepthFunc(GL_GEQUAL);
 		glDepthMask(0);
