@@ -588,10 +588,6 @@ static qboolean VID_SetMode (int width, int height, int bpp, qboolean fullscreen
 		}
 		if (!draw_context)
 			Sys_Error ("Couldn't create window");
-
-		gl_context = SDL_GL_CreateContext (draw_context);
-		if (!gl_context)
-			Sys_Error ("Couldn't create GL context");
 	}
 
 	/* Ensure the window is not fullscreen */
@@ -618,6 +614,13 @@ static qboolean VID_SetMode (int width, int height, int bpp, qboolean fullscreen
 
 	SDL_ShowWindow (draw_context);
 
+	/* Create GL context if needed */
+	if (!gl_context) {
+		gl_context = SDL_GL_CreateContext(draw_context);
+		if (!gl_context)
+			Sys_Error("Couldn't create GL context");
+	}
+
 	gl_swap_control = true;
 	if (SDL_GL_SetSwapInterval ((vid_vsync.value) ? 1 : 0) == -1)
 		gl_swap_control = false;
@@ -734,14 +737,18 @@ static void VID_Restart (void)
 		return;
 	}
 	
-// ericw -- depending on platform / SDL version, after a video mode change we
-// can have a new context (all textures are already freed) or the same context
-// as before, in which case we need to delete the old textures to avoid a
-// memory leak.
+// ericw -- OS X, SDL1: textures, VBO's invalid after mode change
+//          OS X, SDL2: still valid after mode change
+// To handle both cases, delete all GL objects (textures, VBO, GLSL) now.
+// We must not interleave deleting the old objects with creating new ones, because
+// one of the new objects could be given the same ID as an invalid handle
+// which is later deleted.
 
 	TexMgr_DeleteTextureObjects ();
 	GLSLGamma_DeleteTexture ();
 	R_DeleteShaders ();
+	GL_DeleteBModelVertexBuffer ();
+	GLMesh_DeleteVertexBuffers ();
 
 //
 // set new mode
@@ -750,9 +757,10 @@ static void VID_Restart (void)
 
 	GL_Init ();
 	TexMgr_ReloadImages ();
-	GL_BuildVBOs ();
+	GL_BuildBModelVertexBuffer ();
 	GLMesh_LoadVertexBuffers ();
 	GL_SetupState ();
+	Fog_SetupState ();
 
 	//warpimages needs to be recalculated
 	TexMgr_RecalcWarpImageSize ();
@@ -1160,7 +1168,7 @@ static void GL_CheckExtensions (void)
 	//
 	if (COM_CheckParm("-noglslgamma"))
 		Con_Warning ("GLSL gamma disabled at command line\n");
-	else if (gl_glsl_able && gl_texture_NPOT)
+	else if (gl_glsl_able)
 	{
 		gl_glsl_gamma_able = true;
 	}
@@ -1641,7 +1649,13 @@ void	VID_Init (void)
 // new proc by S.A., called by alt-return key binding.
 void	VID_Toggle (void)
 {
-	static qboolean vid_toggle_works = true;
+	// disabling the fast path completely because SDL_SetWindowFullscreen was changing
+	// the window size on SDL2/WinXP and we weren't set up to handle it. --ericw
+	//
+	// TODO: Clear out the dead code, reinstate the fast path using SDL_SetWindowFullscreen
+	// inside VID_SetMode, check window size to fix WinXP issue. This will
+	// keep all the mode changing code in one place.
+	static qboolean vid_toggle_works = false;
 	qboolean toggleWorked;
 	Uint32 flags = 0;
 
