@@ -1,6 +1,7 @@
 /*
 Copyright (C) 1996-2001 Id Software, Inc.
 Copyright (C) 2002-2009 John Fitzgibbons and others
+Copyright (C) 2010-2014 QuakeSpasm developers
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -298,6 +299,7 @@ void M_Main_Key (int key)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		m_entersound = true;
 
 		switch (m_main_cursor)
@@ -378,6 +380,7 @@ void M_SinglePlayer_Key (int key)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		m_entersound = true;
 
 		switch (m_singleplayer_cursor)
@@ -515,6 +518,7 @@ void M_Load_Key (int k)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		S_LocalSound ("misc/menu2.wav");
 		if (!loadable[load_cursor])
 			return;
@@ -558,6 +562,7 @@ void M_Save_Key (int k)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		m_state = m_none;
 		IN_Activate();
 		key_dest = key_game;
@@ -639,6 +644,7 @@ void M_MultiPlayer_Key (int key)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		m_entersound = true;
 		switch (m_multiplayer_cursor)
 		{
@@ -726,8 +732,6 @@ void M_Setup_Draw (void)
 
 void M_Setup_Key (int k)
 {
-	int			l;
-
 	switch (k)
 	{
 	case K_ESCAPE:
@@ -769,6 +773,7 @@ forward:
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		if (setup_cursor == 0 || setup_cursor == 1)
 			return;
 
@@ -799,28 +804,6 @@ forward:
 				setup_myname[strlen(setup_myname)-1] = 0;
 		}
 		break;
-
-	default:
-		if (k < 32 || k > 127)
-			break;
-		if (setup_cursor == 0)
-		{
-			l = strlen(setup_hostname);
-			if (l < 15)
-			{
-				setup_hostname[l+1] = 0;
-				setup_hostname[l] = k;
-			}
-		}
-		if (setup_cursor == 1)
-		{
-			l = strlen(setup_myname);
-			if (l < 15)
-			{
-				setup_myname[l+1] = 0;
-				setup_myname[l] = k;
-			}
-		}
 	}
 
 	if (setup_top > 13)
@@ -833,6 +816,38 @@ forward:
 		setup_bottom = 13;
 }
 
+
+void M_Setup_Char (int k)
+{
+	int l;
+
+	switch (setup_cursor)
+	{
+	case 0:
+		l = strlen(setup_hostname);
+		if (l < 15)
+		{
+			setup_hostname[l+1] = 0;
+			setup_hostname[l] = k;
+		}
+		break;
+	case 1:
+		l = strlen(setup_myname);
+		if (l < 15)
+		{
+			setup_myname[l+1] = 0;
+			setup_myname[l] = k;
+		}
+		break;
+	}
+}
+
+
+qboolean M_Setup_TextEntry (void)
+{
+	return (setup_cursor == 0 || setup_cursor == 1);
+}
+
 //=============================================================================
 /* NET MENU */
 
@@ -927,6 +942,7 @@ again:
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		m_entersound = true;
 		M_Menu_LanConfig_f ();
 		break;
@@ -1197,6 +1213,7 @@ void M_Options_Key (int k)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		m_entersound = true;
 		switch (options_cursor)
 		{
@@ -1284,11 +1301,8 @@ const char *bindnames[][2] =
 
 #define	NUMCOMMANDS	(sizeof(bindnames)/sizeof(bindnames[0]))
 
-#define KEYS_SIZE 18
-
 static int	keys_cursor;
-qboolean	m_keys_bind_grab;
-static int	keys_top;
+static qboolean	bind_grab;
 
 void M_Menu_Keys_f (void)
 {
@@ -1339,7 +1353,7 @@ void M_UnbindCommand (const char *command)
 		if (!b)
 			continue;
 		if (!strncmp (b, command, l) )
-			Key_SetBinding (j, "");
+			Key_SetBinding (j, NULL);
 	}
 }
 
@@ -1355,27 +1369,19 @@ void M_Keys_Draw (void)
 	p = Draw_CachePic ("gfx/ttl_cstm.lmp");
 	M_DrawPic ( (320-p->width)/2, 4, p);
 
-	if (m_keys_bind_grab)
+	if (bind_grab)
 		M_Print (12, 32, "Press a key or button for this action");
 	else
 		M_Print (18, 32, "Enter to change, backspace to clear");
 
-	if (keys_top)
-		M_DrawTransPic (6, 48, pic_up);
-	if (keys_top + KEYS_SIZE < (int)NUMCOMMANDS)
-		M_DrawTransPic (6, 48 + ((KEYS_SIZE-1)*8), pic_down);
-
 // search for known bindings
-	for (i = 0; i < KEYS_SIZE; i++)
+	for (i = 0; i < (int)NUMCOMMANDS; i++)
 	{
-		if (i+keys_top >= (int)NUMCOMMANDS)
-			break;
-
 		y = 48 + 8*i;
 
-		M_Print (16, y, bindnames[i+keys_top][1]);
+		M_Print (16, y, bindnames[i][1]);
 
-		M_FindKeysForCommand (bindnames[i+keys_top][0], keys);
+		M_FindKeysForCommand (bindnames[i][0], keys);
 
 		if (keys[0] == -1)
 		{
@@ -1394,10 +1400,10 @@ void M_Keys_Draw (void)
 		}
 	}
 
-	if (m_keys_bind_grab)
-		M_DrawCharacter (130, 48 + (keys_cursor-keys_top)*8, '=');
+	if (bind_grab)
+		M_DrawCharacter (130, 48 + keys_cursor*8, '=');
 	else
-		M_DrawCharacter (130, 48 + (keys_cursor-keys_top)*8, 12+((int)(realtime*4)&1));
+		M_DrawCharacter (130, 48 + keys_cursor*8, 12+((int)(realtime*4)&1));
 }
 
 
@@ -1406,7 +1412,7 @@ void M_Keys_Key (int k)
 	char	cmd[80];
 	int		keys[2];
 
-	if (m_keys_bind_grab)
+	if (bind_grab)
 	{	// defining a key
 		S_LocalSound ("misc/menu1.wav");
 		if ((k != K_ESCAPE) && (k != '`'))
@@ -1415,7 +1421,7 @@ void M_Keys_Key (int k)
 			Cbuf_InsertText (cmd);
 		}
 
-		m_keys_bind_grab = false;
+		bind_grab = false;
 		IN_Deactivate(modestate == MS_WINDOWED); // deactivate because we're returning to the menu
 		return;
 	}
@@ -1443,25 +1449,21 @@ void M_Keys_Key (int k)
 		break;
 
 	case K_ENTER:		// go into bind mode
+	case K_KP_ENTER:
 		M_FindKeysForCommand (bindnames[keys_cursor][0], keys);
 		S_LocalSound ("misc/menu2.wav");
 		if (keys[1] != -1)
 			M_UnbindCommand (bindnames[keys_cursor][0]);
-		m_keys_bind_grab = true;
+		bind_grab = true;
 		IN_Activate(); // activate to allow mouse key binding
 		break;
 
-	case K_BACKSPACE:		// delete bindings
-	case K_DEL:				// delete bindings
+	case K_BACKSPACE:	// delete bindings
+	case K_DEL:
 		S_LocalSound ("misc/menu2.wav");
 		M_UnbindCommand (bindnames[keys_cursor][0]);
 		break;
 	}
-
-	if (keys_cursor < keys_top)
-		keys_top = keys_cursor;
-	else if (keys_cursor >= keys_top+KEYS_SIZE)
-		keys_top = keys_cursor - KEYS_SIZE + 1;
 }
 
 //=============================================================================
@@ -1555,10 +1557,28 @@ void M_Menu_Quit_f (void)
 
 
 void M_Quit_Key (int key)
+{
+	if (key == K_ESCAPE)
+	{
+		if (wasInMenus)
+		{
+			m_state = m_quit_prevstate;
+			m_entersound = true;
+		}
+		else
+		{
+			IN_Activate();
+			key_dest = key_game;
+			m_state = m_none;
+		}
+	}
+}
+
+
+void M_Quit_Char (int key)
 {
 	switch (key)
 	{
-	case K_ESCAPE:
 	case 'n':
 	case 'N':
 		if (wasInMenus)
@@ -1574,8 +1594,8 @@ void M_Quit_Key (int key)
 		}
 		break;
 
-	case 'Y':
 	case 'y':
+	case 'Y':
 		IN_Deactivate(modestate == MS_WINDOWED);
 		key_dest = key_console;
 		Host_Quit_f ();
@@ -1587,10 +1607,17 @@ void M_Quit_Key (int key)
 
 }
 
+
+qboolean M_Quit_TextEntry (void)
+{
+	return true;
+}
+
+
 void M_Quit_Draw (void) //johnfitz -- modified for new quit message
 {
 	char	msg1[40];
-	char	msg2[] = "by Ozkan Sezer & Stevenaaus"; /* msg2/msg3 are mostly [40] */
+	char	msg2[] = "by Ozkan, Ericw & Stevenaaus"; /* msg2/msg3 are mostly [40] */
 	char	msg3[] = "Press y to quit";
 	int		boxlen;
 
@@ -1602,7 +1629,7 @@ void M_Quit_Draw (void) //johnfitz -- modified for new quit message
 		m_state = m_quit;
 	}
 
-	sprintf(msg1, "QuakeSpasm %1.2f.%d", (float)FITZQUAKE_VERSION, QUAKESPASM_VER_PATCH);
+	sprintf(msg1, "QuakeSpasm %1.2f.%d", (float)QUAKESPASM_VERSION, QUAKESPASM_VER_PATCH);
 
 	//okay, this is kind of fucked up.  M_DrawTextBox will always act as if
 	//width is even. Also, the width and lines values are for the interior of the box,
@@ -1735,6 +1762,7 @@ void M_LanConfig_Key (int key)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		if (lanConfig_cursor == 0)
 			break;
 
@@ -1779,32 +1807,6 @@ void M_LanConfig_Key (int key)
 				lanConfig_joinname[strlen(lanConfig_joinname)-1] = 0;
 		}
 		break;
-
-	default:
-		if (key < 32 || key > 127)
-			break;
-
-		if (lanConfig_cursor == 2)
-		{
-			l = strlen(lanConfig_joinname);
-			if (l < 21)
-			{
-				lanConfig_joinname[l+1] = 0;
-				lanConfig_joinname[l] = key;
-			}
-		}
-
-		if (key < '0' || key > '9')
-			break;
-		if (lanConfig_cursor == 0)
-		{
-			l = strlen(lanConfig_portname);
-			if (l < 5)
-			{
-				lanConfig_portname[l+1] = 0;
-				lanConfig_portname[l] = key;
-			}
-		}
 	}
 
 	if (StartingGame && lanConfig_cursor == 2)
@@ -1823,6 +1825,40 @@ void M_LanConfig_Key (int key)
 	sprintf(lanConfig_portname, "%u", lanConfig_port);
 }
 
+
+void M_LanConfig_Char (int key)
+{
+	int l;
+
+	switch (lanConfig_cursor)
+	{
+	case 0:
+		if (key < '0' || key > '9')
+			return;
+		l = strlen(lanConfig_portname);
+		if (l < 5)
+		{
+			lanConfig_portname[l+1] = 0;
+			lanConfig_portname[l] = key;
+		}
+		break;
+	case 2:
+		l = strlen(lanConfig_joinname);
+		if (l < 21)
+		{
+			lanConfig_joinname[l+1] = 0;
+			lanConfig_joinname[l] = key;
+		}
+		break;
+	}
+}
+
+
+qboolean M_LanConfig_TextEntry (void)
+{
+	return (lanConfig_cursor == 0 || lanConfig_cursor == 2);
+}
+
 //=============================================================================
 /* GAME OPTIONS MENU */
 
@@ -2250,6 +2286,7 @@ void M_GameOptions_Key (int key)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		S_LocalSound ("misc/menu2.wav");
 		if (gameoptions_cursor == 0)
 		{
@@ -2405,6 +2442,7 @@ void M_ServerList_Key (int k)
 		break;
 
 	case K_ENTER:
+	case K_KP_ENTER:
 		S_LocalSound ("misc/menu2.wav");
 		m_return_state = m_state;
 		m_return_onerror = true;
@@ -2625,6 +2663,41 @@ void M_Keydown (int key)
 }
 
 
+void M_Charinput (int key)
+{
+	switch (m_state)
+	{
+	case m_setup:
+		M_Setup_Char (key);
+		return;
+	case m_quit:
+		M_Quit_Char (key);
+		return;
+	case m_lanconfig:
+		M_LanConfig_Char (key);
+		return;
+	default:
+		return;
+	}
+}
+
+
+qboolean M_TextEntry (void)
+{
+	switch (m_state)
+	{
+	case m_setup:
+		return M_Setup_TextEntry ();
+	case m_quit:
+		return M_Quit_TextEntry ();
+	case m_lanconfig:
+		return M_LanConfig_TextEntry ();
+	default:
+		return false;
+	}
+}
+
+
 void M_ConfigureNetSubsystem(void)
 {
 // enable/disable net systems to match desired config
