@@ -1,32 +1,30 @@
 /*
-	q_stdinc.h
-	includes the minimum necessary stdc headers,
-	defines common and / or missing types.
-	NOTE:	for net stuff use net_sys.h,
-		for byte order use q_endian.h,
-		for math stuff use mathlib.h.
-
-	Copyright (C) 1996-1997  Id Software, Inc.
-	Copyright (C) 2007-2011  O.Sezer <sezero@users.sourceforge.net>
-
-	This program is free software; you can redistribute it and/or
-	modify it under the terms of the GNU General Public License
-	as published by the Free Software Foundation; either version 2
-	of the License, or (at your option) any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-
-	See the GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to:
-
-		Free Software Foundation, Inc.
-		51 Franklin St, Fifth Floor,
-		Boston, MA  02110-1301  USA
-*/
+ * q_stdinc.h - includes the minimum necessary stdc headers,
+ *		defines common and / or missing types.
+ *
+ * NOTE:	for net stuff use net_sys.h,
+ *		for byte order use q_endian.h,
+ *		for math stuff use mathlib.h,
+ *		for locale-insensitive ctype.h functions use q_ctype.h.
+ *
+ * Copyright (C) 1996-1997  Id Software, Inc.
+ * Copyright (C) 2007-2011  O.Sezer <sezero@users.sourceforge.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or (at
+ * your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #ifndef __QSTDINC_H
 #define __QSTDINC_H
@@ -34,6 +32,9 @@
 #include <sys/types.h>
 #include <stddef.h>
 #include <limits.h>
+#ifndef _WIN32 /* others we support without sys/param.h? */
+#include <sys/param.h>
+#endif
 
 /* NOTES on TYPE SIZES:
    Quake/Hexen II engine relied on 32 bit int type size
@@ -62,9 +63,6 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
-#if !defined(_WIN32)
-#include <strings.h>	/* strcasecmp and strncasecmp	*/
-#endif	/* ! PLATFORM_WINDOWS */
 
 /*==========================================================================*/
 
@@ -143,6 +141,7 @@ COMPILE_TIME_ASSERT(qboolean, sizeof(qboolean) == 4);
 /* math */
 typedef float	vec_t;
 typedef vec_t	vec3_t[3];
+typedef vec_t	vec4_t[4];
 typedef vec_t	vec5_t[5];
 typedef int	fixed4_t;
 typedef int	fixed8_t;
@@ -151,6 +150,29 @@ typedef int	fixed16_t;
 
 /*==========================================================================*/
 
+/* MAX_OSPATH (max length of a filesystem pathname, i.e. PATH_MAX)
+ * Note: See GNU Hurd and others' notes about brokenness of this:
+ * http://www.gnu.org/software/hurd/community/gsoc/project_ideas/maxpath.html
+ * http://insanecoding.blogspot.com/2007/11/pathmax-simply-isnt.html */
+
+#if !defined(PATH_MAX)
+/* equivalent values? */
+#if defined(MAXPATHLEN)
+#define PATH_MAX	MAXPATHLEN
+#elif defined(_WIN32) && defined(_MAX_PATH)
+#define PATH_MAX	_MAX_PATH
+#elif defined(_WIN32) && defined(MAX_PATH)
+#define PATH_MAX	MAX_PATH
+#else /* fallback */
+#define PATH_MAX	1024
+#endif
+#endif	/* PATH_MAX */
+
+#define MAX_OSPATH	PATH_MAX
+
+/*==========================================================================*/
+
+/* missing types */
 #if defined(_MSC_VER)
 #if defined(_WIN64)
 #define ssize_t	SSIZE_T
@@ -159,12 +181,7 @@ typedef int	ssize_t;
 #endif	/* _WIN64 */
 #endif	/* _MSC_VER */
 
-/* compatibility with M$ types */
-#if !defined(_WIN32)
-#define	PASCAL
-#define	FAR
-#define	APIENTRY
-#endif	/* ! WINDOWS */
+/*==========================================================================*/
 
 #if !defined(__GNUC__)
 #define	__attribute__(x)
