@@ -36,8 +36,6 @@ float	r_avertexnormals[NUMVERTEXNORMALS][3] =
 #include "anorms.h"
 };
 
-vec3_t	shadevector;
-
 extern vec3_t	lightcolor; //johnfitz -- replaces "float shadelight" for lit support
 
 // precalculated dot products for quantized angles
@@ -51,6 +49,7 @@ extern	vec3_t			lightspot;
 
 float	*shadedots = r_avertexnormal_dots[0];
 vec3_t	shadevector;
+
 float	entalpha; //johnfitz
 
 qboolean	overbright; //johnfitz
@@ -86,9 +85,6 @@ static const GLint pose2VertexAttrIndex = 2;
 static const GLint pose2NormalAttrIndex = 3;
 static const GLint texCoordsAttrIndex = 4;
 
-extern GLuint r_meshvbo;
-extern GLuint r_meshindexesvbo;
-
 /*
 =============
 GLARB_GetXYZOffset
@@ -238,8 +234,8 @@ void GL_DrawAliasFrame_GLSL (aliashdr_t *paliashdr, lerpdata_t lerpdata, gltextu
 
 	GL_UseProgramFunc (r_alias_program);
 
-	GL_BindBuffer (GL_ARRAY_BUFFER, r_meshvbo);
-	GL_BindBuffer (GL_ELEMENT_ARRAY_BUFFER, r_meshindexesvbo);
+	GL_BindBuffer (GL_ARRAY_BUFFER, currententity->model->meshvbo);
+	GL_BindBuffer (GL_ELEMENT_ARRAY_BUFFER, currententity->model->meshindexesvbo);
 
 	GL_EnableVertexAttribArrayFunc (texCoordsAttrIndex);
 	GL_EnableVertexAttribArrayFunc (pose1VertexAttrIndex);
@@ -412,7 +408,7 @@ void R_SetupAliasFrame (aliashdr_t *paliashdr, int frame, lerpdata_t *lerpdata)
 
 	if ((frame >= paliashdr->numframes) || (frame < 0))
 	{
-		Con_DPrintf ("R_AliasSetupFrame: no such frame %d\n", frame);
+		Con_DPrintf ("R_AliasSetupFrame: no such frame %d for '%s'\n", frame, e->model->name);
 		frame = 0;
 	}
 
@@ -686,7 +682,7 @@ void R_DrawAliasModel (entity_t *e)
 	anim = (int)(cl.time*10) & 3;
 	if ((e->skinnum >= paliashdr->numskins) || (e->skinnum < 0))
 	{
-		Con_DPrintf ("R_DrawAliasModel: no such skin # %dn", e->skinnum);
+		Con_DPrintf ("R_DrawAliasModel: no such skin # %d for '%s'\n", e->skinnum, e->model->name);
 		tx = NULL; // NULL will give the checkerboard texture
 		fb = NULL;
 	}
